﻿
using System;
using System.Xml;
using HeuristicLab.Algorithms.GraphRouting;
using HeuristicLab.Problems.RoutePlanning.Graph;
using HeuristicLab.Problems.RoutePlanning.Osm;
using HeuristicLab.Problems.RoutePlanning.Osm.Data;
namespace HeuristicLab.Problems.RoutePlanning.Test {
  class Program {
    static void Main(string[] args) {
      Test1();
    }

    private static void Test1() {
      string filepath = @"..\..\OsmTestFiles\test_mid.osm";

      Console.Write("Loading data ... ");
      XmlDataSource xmlDs = new XmlDataSource(filepath);
      Console.WriteLine("done.");

      Graph.Graph graph = new Graph.Graph(xmlDs);
      IRouter router = new DijkstraAlgorithm(graph);

      Console.Write("Calculate route ... ");
      long[] result = router.Calculate(529102170, 1001363194);
      System.Console.WriteLine("done.");

      Console.Write("Result: ");
      foreach (long i in result) System.Console.Write(i + "; ");
      Console.WriteLine();

      WriteGPXFile(graph, result);

      System.Console.Read();
    }

    private static void Test2() {
      //string filepath = @"..\..\OsmTestFiles\test.osm";
      //string filepath = @"..\..\OsmTestFiles\testNode1.osm";
      //string filepath = @"..\..\OsmTestFiles\testWay1.osm";
      //string filepath = @"..\..\OsmTestFiles\testRelation1.osm";
      string filepath = @"..\..\OsmTestFiles\test_mid.osm";

      Console.Write("Loading data ... ");
      XmlDataSource xmlDs = new XmlDataSource(filepath);
      Console.WriteLine("done.");

      Graph.Graph graph = new Graph.Graph(xmlDs);
      IRouter router = new DijkstraAlgorithm(graph);

      Console.Write("Calculate route ... ");
      long[] result = router.Calculate(529102170, 1001363194);
      System.Console.WriteLine("done.");

      Console.Write("Result: ");
      foreach (long i in result) System.Console.Write(i + "; ");
      Console.WriteLine();
      System.Console.Read();
    }

    private static void WriteGPXFile(Graph.Graph graph, long[] route) {
      XmlWriterSettings settings = new XmlWriterSettings();
      settings.Indent = true;
      settings.IndentChars = ("   ");

      using (XmlWriter writer = XmlWriter.Create("RoutePath.gpx", settings)) {
        writer.WriteStartElement("gpx", "http://www.topografix.com/GPX/1/0");
        writer.WriteAttributeString("version", "1.1");
        writer.WriteAttributeString("creator", "cloudia");

        foreach (long nodeId in route) {
          Vertex<Node> node = graph.GetVertex(nodeId);
          writer.WriteStartElement("wpt");
          writer.WriteAttributeString("lat", XmlConvert.ToString(node.Node.Latitude));
          writer.WriteAttributeString("lon", XmlConvert.ToString(node.Node.Longitude));
          writer.WriteEndElement(); // wpt
        }

        //writer.WriteStartElement("rte");
        //foreach (long nodeId in route) {
        //  Vertex<Node> node = graph.GetVertex(nodeId);
        //  writer.WriteStartElement("rtept");
        //  writer.WriteAttributeString("lat", XmlConvert.ToString(node.Node.Latitude));
        //  writer.WriteAttributeString("lon", XmlConvert.ToString(node.Node.Longitude));
        //  writer.WriteEndElement(); // rtept
        //}
        //writer.WriteEndElement(); // rte

        //writer.WriteStartElement("trk");
        //writer.WriteElementString("name", "RoutePath");
        ////writer.WriteEndElement(); // name
        //writer.WriteStartElement("trkseq");
        //foreach (long nodeId in route) {
        //  Vertex<Node> node = graph.GetVertex(nodeId);
        //  writer.WriteStartElement("trkpt");
        //  writer.WriteAttributeString("lat", XmlConvert.ToString(node.Node.Latitude));
        //  writer.WriteAttributeString("lon", XmlConvert.ToString(node.Node.Longitude));
        //  writer.WriteEndElement(); // trkpt
        //}
        //writer.WriteEndElement(); // trkseq
        //writer.WriteEndElement(); // trk

        writer.WriteEndElement(); // gpx
      }
    }
  }
}
